#include <stdio.h>
#include <math.h>
#include "path.h"


int check_hbfirst();

double energy(Slice *psl) 
{
  double x, y, x2, y2, v;

  x = psl->x;
  y = psl->y;
    
  x2 = x*x;
  y2 = y*y;
 

  v = (2./15) *exp (-2 - 4 * y) -
    exp (-(x-1)*(x-1)-y2) - exp(-(x+1)*(x+1)-y2) +
    5*exp(-0.32*(x2 + y2 +20*(x+y)*(x+y))) + (32./1875)*(x2*x2 +y2*y2);

  return v;
}


double check_energy(Slice *psl) 
{
  double x, y, x2, y2, v,ke;

  x = psl->x;
  y = psl->y;
    
  x2 = x*x;
  y2 = y*y;
 

  v = (2./15) *exp (-2 - 4 * y) -
    exp (-(x-1)*(x-1)-y2) - exp(-(x+1)*(x+1)-y2) +
    5*exp(-0.32*(x2 + y2 +20*(x+y)*(x+y))) + (32./1875)*(x2*x2 +y2*y2);
  

  ke = 0.5*(psl->vx*psl->vx +psl->vy*psl->vy);
  
  printf("V = %12.7g, K = %12.7g, E= %12.6g\n", v, ke,ke+v);
  
  return v;
}



void forces(double x, double y, double *fx, double *fy) 
{
  double x2, y2, v,xpy2, expo1, expo2, expo3;

  x2 = x*x;
  y2 = y*y;
  xpy2 =(x+y)*(x+y);
  
  expo1 = exp(-(x-1)*(x-1)-y2);
  expo2 = exp(-(x+1)*(x+1)-y2);
  expo3 = exp(-0.32*(x2 + y2 +20*xpy2));

  *fx =-( 2 * (x-1) * expo1+ 
       2 * (x+1) * expo2 - 
     1.6 * expo3 * (2*x+40*(x+y)) +
    (128./1875)*x*x2);
  
    
  *fy = -(2 * y * expo1+ 
       2 * y * expo2 - 
     1.6 * expo3* (2*y+40*(x+y)) +
    (128./1875)*y*y2 -(8./15)*exp(-2- 4*y));
      



  return;
}




int metropolis(Slice *psl)

{

  double dx, dy, energy_old, energy_trial, aux,old_x,old_y;
  int i,hb;


  
  dx = gssran()*sys.sigma;
  dy = gssran()*sys.sigma;
  
  energy_old = psl->energy;  
  old_x = psl->x;
  old_y = psl->y;
  psl->x += dx;
  psl->y += dy;



  energy_trial = energy(psl);
  aux = -sys.beta*(energy_trial-energy_old);
  if (exp(aux) < ran3()) { 
    psl->x =old_x;
    psl->y =old_y;

    return 0;
  }
  psl->energy = energy_trial;
  return 1;
} 

int langevin_highfriction(Slice *psl)

{
  double xnew,ynew,dx,dy,fx,fy;

  forces(psl->x,psl->y,&fx,&fy);

  xnew = psl->x + sys.bDdt*fx;
  ynew = psl->y + sys.bDdt*fy;

  dx = gssran()*sys.sigma;
  dy = gssran()*sys.sigma;
  
  psl->x = xnew + dx;
  psl->y = ynew + dy;
}


int langevin_inertia(Slice *psl)

{
  double xnew,ynew,dx,dy,dvx,dvy,fx,fy;
 

  
  dx=dy=dvx=dvy=0;
  if (langevin.gamma >0) {
      gssbivar(&dx,&dvx);
      gssbivar(&dy,&dvy);
  }

  psl->x  += sys.dt*(langevin.c1*psl->vx + langevin.c2*sys.dt*psl->fx) +dx;
  psl->y  += sys.dt*(langevin.c1*psl->vy + langevin.c2*sys.dt*psl->fy) +dy; 
  psl->vx = langevin.c0 * psl->vx + langevin.c1min2 *sys.dt * psl->fx;
  psl->vy = langevin.c0 * psl->vy + langevin.c1min2 *sys.dt * psl->fy;
  forces(psl->x,psl->y,&psl->fx,&psl->fy);
 
  psl->vx += langevin.c2 * sys.dt * psl->fx +dvx;
  psl->vy += langevin.c2 * sys.dt * psl->fy +dvy;
  
}


void verlet(Slice *psl)
{
  double dt,dt2;

  dt =sys.dt;
  dt2 =dt*0.5;

  psl->vx +=  dt2* psl->fx;
  psl->vy +=  dt2* psl->fy;     
  psl->x  +=  dt* psl->vx;
  psl->y  +=  dt* psl->vy;
  forces(psl->x,psl->y,&psl->fx,&psl->fy);
  psl->vx +=  dt2* psl->fx;
  psl->vy +=  dt2* psl->fy;
  
  return;
}


int in_stable_state_B_z(Slice *psl)

{
  double x,y,z,dist2;
  int inB,not_in_A;

  z =psl->x+ psl->y;
  
  inB = (z> path.lower_bound_B) && (z < path.upper_bound_B);
  return inB;
}

int in_stable_state_B(Slice *psl)

{
  double x,y,dist,tana,angle;
  int inB,not_in_A;

  /*  if (psl->x>0) return 1;*/

  x =psl->x- path.final.x;
  y= psl->y- path.final.y;  
  tana = y/x;
  angle = atan(tana);
  if (x<0) angle+=PI;
  inB = (angle > path.lower_bound_B) && (angle < path.upper_bound_B);
  
  return inB ;
}

int in_stable_state_A(Slice *psl)

{
  double x,y,dist,tana,angle;
  int inA;


  x =psl->x - path.initial.x;
  y= psl->y - path.initial.y;
  tana = y/x;
  angle = atan(tana);
  if (x<0) angle+=PI;
  inA = (angle > path.lower_bound_A) && (angle < path.upper_bound_A);
  
  return inA ;
}


int in_stable_state_A_z(Slice *psl)
{
  double x,y,z,dist2;
  int inA;

  z =psl->x + psl->y; 
  inA = (z < path.upper_bound_A);
 
  return inA;
}

int number_of_slices_in_B()
{


  int i;

  for(i=0;i<path.nslices;i++){
    if (in_stable_state_B(&slice[i]))  return path.nslices-i;
  }
  return -1;
}


int number_of_slices_in_A()

{
  int i;
  
  for(i=path.nslices-1;i>=0;i--){
    if (in_stable_state_A(&slice[i]))  return i;
  }
  return -1;
}


  
int check_hbfirst()
{
  int i,hbfirst=0;

  for(i=0;i<path.nslices;i++) {
    if (in_stable_state_B(&slice[i])) {
      hbfirst = i;
      break;
    }
  }
  if (hbfirst != path.hbfirst) {
    printf("warning: hbfirst corrupted\n");
    dprint(hbfirst);
    dprint(path.hbfirst);
    return 1;

  }
  return 0;
}

int visit_to_B(Slice *trial) 
{  
  int i, in_ring;

  if (path.endpoint_constrained) { 
    if (in_stable_state_B(&trial[path.nslices-1])==0) return 0;
  } else {
    in_ring=0;
    for(i=0;i<path.nslices;i++) {
      if (in_stable_state_B(&trial[i])==1) {
	in_ring=1;
	break;
      }
    }
  if (in_ring==0) return 0;
  } 
  return 1;
}


int shoot_forward()
{
  int i,itime,in_ring,hb_first;
    
  itime = ran3()*(path.nslices);
  scr.pftry[itime]++;

  trial = (path.which_array==0) ? seq_1 : seq_2;
  for(i=0;i<=itime;i++) trial[i] = slice[i]; 
  for (i=itime+1;i<path.nslices;i++) {
    trial[i] = trial[i-1]; 
    langevin_inertia(&trial[i]);
  }

  if (visit_to_B(trial)==0) return 0;

  slice = trial;
  path.which_array = !path.which_array;

  scr.pfacc[itime]++;
  update_average(NFORWARD,path.nslices-itime);
  return 1;
}

int shoot_backward()
{
  int i,itime,in_ring,hb_first;

  itime = ran3()*(path.nslices);
  scr.pbtry[itime]++;
  
  trial = (path.which_array==0) ? seq_1 : seq_2;
  trial[itime] = slice[itime]; 
  trial[itime].vx = -trial[itime].vx;
  trial[itime].vy = -trial[itime].vy;
  for (i=itime-1;i>=0;i--) {
    trial[i] = trial[i+1]; 
    langevin_inertia(&trial[i]);
  }
  if (in_stable_state_A(&trial[0])==0) return 0;
  for(i=itime;i<path.nslices;i++) trial[i]=slice[i];
  
  if (visit_to_B(trial)==0) return 0;

  for (i=0;i<itime;i++) {
    trial[i].vx = -trial[i].vx;
    trial[i].vy = -trial[i].vy;
  }

  slice =trial;
  path.which_array = !path.which_array;
 
  scr.pbacc[itime]++;
  update_average(NBACKWARD,itime); 
  return 1;
}

int reptation_left()
{

  int nmove,new_end,i,in_ring,hb_first;

  nmove =(int)(ran3()*(path.nslices/2) +1);
  scr.pltry[nmove]++;
  
  trial = (path.which_array==0) ? seq_1 : seq_2;
  trial[nmove] = slice[0];
  trial[nmove].vx = -trial[0].vx;
  trial[nmove].vy = -trial[0].vy;
  for(i=nmove-1;i>=0;i--) {
    trial[i] = trial[i+1]; 
    langevin_inertia(&trial[i]); 
  }
  if (in_stable_state_A(&trial[0])==0) return 0;
  
  for (i=nmove;i<path.nslices;i++) trial[i]=slice[i-nmove];

  if (visit_to_B(trial)==0) return 0;
 
  
  for (i=0;i<nmove;i++) {
    trial[i].vx = -trial[i].vx;
    trial[i].vy = -trial[i].vy;
  }
  
  slice =trial;
  path.which_array = !path.which_array;
 
  update_average(NLEFT,nmove);
  scr.placc[nmove]++;
  
  return 1;
}

int reptation_right()
{
  int i,nmove,in_ring;
  
  nmove =(int)(ran3()*(path.nslices/2) +1);
  scr.prtry[nmove]++;
 
  if (in_stable_state_A(&slice[nmove])==0) return 0;
 
  trial = (path.which_array==0) ? seq_1 : seq_2;
  
  for (i=0;i<path.nslices-nmove;i++) {
    trial[i]=slice[i+nmove];
  }
  for(i=path.nslices-nmove;i<path.nslices;i++) {
    trial[i] = trial[i-1]; 
    langevin_inertia(&trial[i]); 
  }

  if (visit_to_B(trial)==0) return 0;
 
  slice =trial;
  path.which_array = !path.which_array;

  update_average(NRIGHT,nmove);
  scr.pracc[nmove]++;
  return 1;
}


int diffuse_left()
{
}

int diffuse_right()
{
}

void save_trialpath(int ifile, Slice *psl)

{
  char filename[40];
  FILE *fp;
  int i,j;  
  
  sprintf(filename,"trial%d",ifile);
  if ((fp = fopen(filename,"w"))==NULL){
    printf("output: can't open %s\n",filename);
    return;
  } else {
    for(i=0;i<path.nslices;i++) {
      in_stable_state_B(&psl[i]);
      fprintf(fp,"%d ",i);
      for(j=0;j<sys.ncrit;j++)  fprintf(fp," %lf ",sys.crit[j]);
      fprintf(fp,"\n");
    }
    fprintf(fp,"*\n");
  }   
  fclose(fp);
}

