/*------------------INLINE SUBSTITUTIONS-------------------------------------*/
#define TRUE 1
#define FALSE 0
#define CANCELLED -99

#define P_INDEX(ptr)  (ptr - pts)
#define F_COMP(a,b) (fabs( a-b ) < 1e-10)
#define MIN(a,b,tmp)    (  (tmp=a) < b  ? tmp : b )
#define MAX(a,b,tmp)    (  (tmp=a) > b  ? tmp : b )
#define MESSAGE(a) printf("Message:"#a"\n")
#define SIGN(a)   ( 2*(a>0) -1) 

#define PI           3.141592653589793
#define SQRT2        1.4142136
#define BIGNUM       1e99
#define EPS          1e-10

#define dprint(expr) printf(#expr " = %d\n",expr)
#define gprint(expr) printf(#expr " = %g\n",expr)
#define vprint(expr) printf(#expr " = ( %16.14g %16.14g %16.14g ) \n",expr.x, expr.y, expr.z)
#define vector_inp(a, b) (a.x * b.x + a.y * b.y + a.z * b.z )
#define vector_cross(a,b,h)   h.x = a.y * b.z - a.z * b.y; h.y = a.z * b.x - a.x * b.z; h.z = a.x * b.y - a.y * b.x;
#define vector_times(a,b,h)   h.x = a.x * b.x; h.y = a.y * b.y; h.z = a.z * b.z;
#define vector_divide(a,b,h)   h.x = a.x / b.x; h.y = a.y / b.y; h.z = a.z / b.z;
#define vector_plustimes(a,b,h)   h.x += a.x * b.x; h.y += a.y * b.y; h.z += a.z * b.z;
#define vector_mintimes(a,b,h)   h.x -= a.x * b.x; h.y -= a.y * b.y; h.z -= a.z * b.z;
#define scalar_times(a,b,h)  h.x = a.x *b; h.y =a.y * b; h.z = a.z * b;  
#define scalar_divide(a,b,h)  h.x = a.x /b; h.y =a.y / b; h.z = a.z / b;  
#define scalar_plustimes(a,b,h)  h.x += a.x *b; h.y +=a.y * b; h.z += a.z * b;  
#define scalar_mintimes(a,b,h)  h.x -= a.x *b; h.y -=a.y * b; h.z -= a.z * b;  
#define vector_add(a,b,h)  {h.x = a.x +b.x; h.y =a.y + b.y; h.z = a.z + b.z;  }
#define vector_minus(a,b,h) { h.x = a.x - b.x; h.y =a.y - b.y; h.z = a.z - b.z;  }
#define update_average(a,b) {a.now =b; a.sum += a.now; a.sumsq += a.now*a.now; a.n++;}
#define SWAP(a,b,c) {c=a;a=b;b=c;}

#define POTENERGY sys.block_stats.aver[0]
#define KIN_ENER  sys.block_stats.aver[1]
#define TOT_ENER  sys.block_stats.aver[2]
#define TEMP      sys.block_stats.aver[3]
#define TEMP_L    sys.block_stats.aver[4]
#define HAHB      sys.block_stats.aver[5]
#define HB        sys.block_stats.aver[6]
#define BLIP      sys.block_stats.aver[7]

#define NSLA      sys.block_stats.shoot[0]
#define NSLB      sys.block_stats.shoot[1]
#define NSLBAR    sys.block_stats.shoot[2]
#define NFORWARD  sys.block_stats.shoot[3]
#define NBACKWARD sys.block_stats.shoot[4]
#define NDIFF     sys.block_stats.shoot[5]
#define NLEFT     sys.block_stats.shoot[6]
#define NRIGHT    sys.block_stats.shoot[7]
#define BEGIN1    sys.block_stats.shoot[8]
#define END1      sys.block_stats.shoot[9]

#define FORWARD_ACC      sys.block_stats.mcacc[0]
#define BACKWARD_ACC     sys.block_stats.mcacc[1]
#define REPTLEFT_ACC     sys.block_stats.mcacc[2]
#define REPTRIGHT_ACC    sys.block_stats.mcacc[3]
#define DIFLEFT_ACC      sys.block_stats.mcacc[4]
#define DIFRIGHT_ACC     sys.block_stats.mcacc[5]
#define INREGIONB        sys.block_stats.mcacc[6]



#define NSTAT  8
#define NACC    7
#define NSHOOTAV 10
#define MBIN  500
#define MAXRUN 50
/*---------------------------------------------------------------------------*/
/*------------------STRUCTURE DEFINITIONS------------------------------------*/

typedef struct vector_type {

      double         x,y,z;

} vector;


typedef struct tensor_type {

      vector         x,y,z;

} tensor;



typedef struct particle_type *Pts_ptr;

typedef struct particle_type {

      double        x                 ,
                    y                 ,
                    vx                ,
                    vy                ,
                    fx                ,
                    fy                ,
                    energy            ;

      int           ha                ,
                    hb                ;
	
} Slice;


typedef struct aver_type {

      char          *name              ;

      int           n                  ;

      double        now                ,
                    sum                ,
                    sumsq              ;

} Average;

typedef struct mcacc_type {

      char          *name              ;

      int           acc,try             ;

      double        ratio              ;

} Mcacc;



typedef struct stats_type {

      Average       aver[NSTAT];

      Average       shoot[NSHOOTAV];       

      Mcacc         mcacc[NACC];

} Stats;

typedef struct corr_type {
  
      double        free_en[MBIN]         ,
                    hadhb[MBIN]           ,
                    hahb[MBIN]            ,
                    haHb[MBIN]            ,
                    blip[MBIN]            ,
	            xdist[MBIN]           ,
	            ydist[MBIN]           ,
	            

                    dHahbt[MBIN]          ,
                    dHaHbt[MBIN]          ,
                    dHahat[MBIN]          ,
                    dHaHat[MBIN]          ,                   
                    energy[MBIN]          ,
       
                    energy2[MBIN]         ,
                    kin_ener[MBIN]        ,
                    kin_ener2[MBIN]       ,
                    tot_ener[MBIN]        ,
                    tot_ener2[MBIN]       ;
       
      int           navb[MBIN]             ,
                    nava[MBIN]             ,    
                    pfacc[MBIN]            ,
                    pbacc[MBIN]            ,
                    placc[MBIN]            ,
                    pracc[MBIN]            ,
                    pftry[MBIN]            ,
                    pbtry[MBIN]            ,
                    pltry[MBIN]            ,
                    prtry[MBIN]            ,
                    palign_acc[MBIN]       ,
                    palign_try[MBIN]       ,
                    moi1[MBIN]             ,
                    endpointdist[MBIN]     ,
                    tp[MBIN]               ,

	            fcount                ,
                    n                     ;
      double        free_ch               ;
} Scr;


typedef struct bivar_type {
      double        sig_r              ,
                    sig_v              ,
                    sigma              ,
                    c_rv               ,
                    s12os11            ,
                    sqrts11            ,
                    sqrtSos11          ,
                    norm               ,
                    norder             ,
                    pfmax              ,
                    pbmax              ,
                    pb[10]             ,
                    pf[10]             ;


} Bivar;

typedef struct langevin_type {
  
  
     double         gamma              ,
                    gamma_a            ,
                    gamma_b            ,
	            c0                 ,
	            c1                 ,
	            c2                 ,
	            c1min2             ;

} Langevin;

typedef struct path_typ {
      int           nslices            ,
	            hbfirst            ,
                    quench             ,
	            endpoint_constrained,
                    which_array        ;

      double        energy             ,
                    action             ,
                    kin_ener           ,
	            tot_ener           ,
	            lower_bound_A      ,
	            upper_bound_A      ,
	            lower_bound_B      ,
	            upper_bound_B      ,
                    order_par          ,
                    ring_dist2         ,
	            final_width        ,
                    divide             ;

      vector        region             ,
	            initial            ,
                    final              ;

      

} Path;  





typedef struct parameter_type {

      int           ncycle1            ,
                    ncycle2            ,
                    npart              ,
	            nshoot             ,
                    nrept              ,
	            ndif               ,
                    nacc               ,
                    maxtrial           ,
                    block_no           ,
                    sim_type           ,
                    start_type         ,
       	            freq_check         ,
                    freq_graphics      ,
                    freq_config        ,
	            freq_corr          ,
	            freq_dump          ,
                    graphics           ,
	            hAmin[5]           ,
	            hAmax[5]           ,
	            hBmin[5]           ,
	            hBmax[5]           ,
	            ncrit              ,                    
	            vers               ;

      double        volume             ,
                    energy             ,
                    beta               ,
                    temp               ,
                    dt                 ,
                    bDdt               ,
                    maxdist            ,
                    potw               ,
	            saddlex            ,
                    saddley            ,
	            sigma              , 
                    norm               ,
                    trial              ,
	            crit[5]            ,
	            initx              ,
                    inity              ;
                    
      

      Stats         final_stats        ,
                    block_stats        ;


} Sys;

                    
/*---------------------------------------------------------------------------*/
/*------------------GLOBALLY DEFINED VARIABLES-------------------------------*/

extern Slice         *slice,*trial,*seq_1,*seq_2;
extern Sys           sys;
extern Path          path;
extern Langevin      langevin;
extern Stats         nulstat;
extern vector        nulvec;
extern Scr           scr,nulscr;
extern Bivar         bivar              ;



/*---------------------------------------------------------------------------*/
/*------------------GLOBALLY DEFINED FUNCTIONS-------------------------------*/






extern void  set_up             ( void );
extern void  confinput          ( void );
extern void  confoutput         ( void );
extern void  distoutput         ( void );
extern double  energy           ( Slice * );
extern double check_energy      ( Slice *) ;
extern int   langevin_inertia    (Slice *);
extern double ran3              ( void );
extern double gssran            ( void );
extern void   gssbivar          (double *, double *);
extern void  plot_conf          ( void );
extern void  init_graphics      ( void );
extern void  drawline( vector,vector);
extern int   in_stable_state_A(Slice *);
extern int   in_stable_state_B(Slice *);
extern int   reptation_left();
extern int   reptation_right();
extern int   diffuse_left();
extern int   diffuse_right();

